"use strict";
/** 根据配置生成匹配正则 */
function createFilter(roots, folders) {
    const rootsStr = roots.map((r) => {
        let res = r.name;
        for (let i = 0; i < r.depth; i++) {
            res += "/[^/]+";
        }
        return res;
    });
    return {
        fileFilter: new RegExp(`/(${rootsStr.join("|")})/`),
        importFilter: new RegExp(`^@/(${folders.join("|")})\\b`),
    };
}
const extensions = [".ts", ".tsx", ".js", ".jsx"];
function init(modules) {
    const ts = modules.typescript;
    /** 自解析模块缓存 */
    const cache = {};
    function resolvedModule(moduleName, containingFile, fileFilter) {
        const m = containingFile.match(fileFilter);
        const module = containingFile.slice(0, m.index + m[0].length) +
            moduleName.replace("@/", "");
        if (!cache[module]) {
            let filename = "";
            if (/\/$/.test(module)) {
                for (const ext of extensions) {
                    if (ts.sys.fileExists(`${module}index${ext}`)) {
                        filename = `${module}index${ext}`;
                        break;
                    }
                }
            }
            else {
                for (const ext of extensions) {
                    if (ts.sys.fileExists(`${module}${ext}`)) {
                        filename = `${module}${ext}`;
                        break;
                    }
                }
                if (!filename) {
                    for (const ext of extensions) {
                        if (ts.sys.fileExists(`${module}/index${ext}`)) {
                            filename = `${module}/index${ext}`;
                            break;
                        }
                    }
                }
            }
            if (filename) {
                cache[module] = filename;
            }
        }
        return cache[module] ? { resolvedFileName: cache[module] } : undefined;
    }
    function create(info) {
        /** 日志输出 */
        function log(...args) {
            info.project.projectService.logger.info(args.map((arg) => String(arg)).join(" "));
        }
        log("Typescript-dynamic-path-plugin started!");
        // 根目录
        const roots = info.config.roots || [
            {
                name: "ACT",
                depth: 2,
            },
            {
                name: "COMMON",
                depth: 1,
            },
        ];
        // 相对目录
        const folders = info.config.folders || [
            "api",
            "components",
            "pages",
            "store",
            "img",
            "js",
            "style",
            "scripts",
        ];
        if ((roots === null || roots === void 0 ? void 0 : roots.length) && (folders === null || folders === void 0 ? void 0 : folders.length)) {
            const { fileFilter, importFilter } = createFilter(roots, folders);
            if (info.languageServiceHost.resolveModuleNames) {
                const _resolveModuleNames = info.languageServiceHost.resolveModuleNames.bind(info.languageServiceHost);
                info.languageServiceHost.resolveModuleNames = (moduleNames, containingFile, ...rest) => {
                    let resolvedModules = _resolveModuleNames(moduleNames, containingFile, ...rest);
                    resolvedModules = resolvedModules.map((m, index) => {
                        if (!m &&
                            importFilter.test(moduleNames[index]) &&
                            fileFilter.test(containingFile) &&
                            !/\.[^\\/.]+$/.test(moduleNames[index])) {
                            return resolvedModule(moduleNames[index], containingFile, fileFilter);
                        }
                        return m;
                    });
                    return resolvedModules;
                };
            }
        }
        return info.languageService;
    }
    return { create };
}
module.exports = init;
//# sourceMappingURL=index.js.map